@@ -13,6 +13,7 @@
 #include <stdio.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <unistd.h>
 
 #include "ui.h"
 #include "scanner.h"
@@ -75,6 +76,17 @@ update_scan_devices_cb (Scanner *scanner, GList *devices)
 }
 
 
+static void
+authorize_cb (Scanner *scanner, const gchar *resource)
+{
+    gchar *username = NULL, *password = NULL;
+    ui_authorize (ui, resource, &username, &password);
+    scanner_authorize (scanner, username, password);
+    g_free (username);
+    g_free (password);
+}
+
+
 static void
 scanner_page_info_cb (Scanner *scanner, ScanPageInfo *info)
 {
@@ -224,66 +236,99 @@ page_removed_cb (Book *book, Page *page, SimpleScan *ui)
 }
 
 
+static gboolean
+save_book (const gchar *uri, GError **error)
+{
+    gboolean result;
+    gchar *uri_lower;
+
+    uri_lower = g_utf8_strdown (uri, -1);
+    if (g_str_has_suffix (uri_lower, ".pdf"))
+        result = book_save_pdf (book, uri, error);
+    else if (g_str_has_suffix (uri_lower, ".ps"))
+        result = book_save_ps (book, uri, error);
+    else if (g_str_has_suffix (uri_lower, ".png"))
+        result = book_save_png (book, uri, error);
+    else
+        result = book_save_jpeg (book, uri, error);
+
+    g_free (uri_lower);
+
+    return result;
+}
+
+
 static void
-save_cb (SimpleScan *ui, gchar *uri)
+save_cb (SimpleScan *ui, const gchar *uri)
 {
-    GFile *file;
     GError *error = NULL;
-    GFileOutputStream *stream;
 
-    file = g_file_new_for_uri (uri);
+    g_debug ("Saving to '%s'", uri);
 
-    stream = g_file_replace (file, NULL, FALSE, G_FILE_CREATE_NONE, NULL, &error);
-    if (!stream) {
+    if (!save_book (uri, &error)) {
         g_warning ("Error saving file: %s", error->message);
+        ui_show_error (ui,
+                       /* Title of error dialog when save failed */
+                       _("Failed to save file"),
+                       error->message);
         g_error_free (error);
     }
-    else {
-        gboolean result;
-        gchar *uri_lower;
-
-        uri_lower = g_utf8_strdown (uri, -1);
-        if (g_str_has_suffix (uri_lower, ".pdf"))
-            result = book_save_pdf (book, stream, &error);
-        else if (g_str_has_suffix (uri_lower, ".ps"))
-            result = book_save_ps (book, stream, &error);
-        else if (g_str_has_suffix (uri_lower, ".png"))
-            result = book_save_png (book, stream, &error);
-        else
-            result = book_save_jpeg (book, stream, &error);
-
-        g_free (uri_lower);           
-
-        if (error) {
-            g_warning ("Error saving file: %s", error->message);
-            ui_show_error (ui,
-                           /* Title of error dialog when save failed */
-                           _("Failed to save file"),
-                           error->message);
-            g_error_free (error);
-        }
-
-        g_output_stream_close (G_OUTPUT_STREAM (stream), NULL, NULL);
-    }
 }
 
 
 static void
 email_cb (SimpleScan *ui)
 {
-    GError *error = NULL;
-    g_spawn_command_line_async ("xdg-email", &error);
-    if (error) {
-        g_warning ("Unable to start email: %s", error->message);
-        g_error_free (error);
+    gint i;
+    gchar *dir, *path = NULL, *uri = NULL;
+
+    // TODO: Delete old files on startup
+
+    /* Save in the temporary dir */
+    dir = g_build_filename (g_get_user_cache_dir (), "simple-scan", "email", NULL);
+    g_mkdir_with_parents (dir, 0700);
+
+    for (i = 0; ; i++) {
+        GString *filename;
+        GError *error = NULL;
+
+        filename = g_string_new ("");
+        g_string_printf (filename, "scan-%d-%d.pdf", getpid (), i);
+
+        g_free (path);
+        path = g_build_filename (dir, filename->str, NULL);
+        g_string_free (filename, TRUE);
+
+        g_free (uri);
+        uri = g_filename_to_uri (path, NULL, NULL);
+
+        if (book_save_pdf (book, uri, &error)) {
+            GString *command_line;
+
+            command_line = g_string_new ("");
+            g_string_printf (command_line, "xdg-email --attach %s", path);
+            g_debug ("Launchind email client: %s", command_line->str);
+            g_spawn_command_line_async (command_line->str, &error);
+
+            if (error) {
+                g_warning ("Unable to start email: %s", error->message);
+                g_clear_error (&error);
+            }
+            g_string_free (command_line, TRUE);
+	    break;
+        }
+        else {
+            if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS)) {
+                g_warning ("Unable to save email PDF: %s", error->message);
+                g_clear_error (&error);
+                break;
+            }
+        }
     }
-}
 
-
-static void
-print_cb (SimpleScan *ui, cairo_t *context)
-{
-    book_print (book, context);
+    g_free (path);
+    g_free (uri);   
+    g_free (dir);
 }
 
 
@@ -317,6 +362,7 @@ usage(int show_gtk)
     fprintf(stderr,
             /* Description on how to use simple-scan displayed on command-line */    
             _("Help Options:\n"
+              "  -d, --debug                     Print debugging messages\n"	      
               "  -v, --version                   Show release version\n"
               "  -h, --help                      Show help options\n"
               "  --help-all                      Show all help options\n"
@@ -340,15 +386,30 @@ usage(int show_gtk)
 }
 
 
+static void
+log_cb (const gchar *log_domain, GLogLevelFlags log_level,
+	const gchar *message, gpointer data)
+{
+    if (log_level & G_LOG_LEVEL_DEBUG)
+       return;
+    g_log_default_handler (log_domain, log_level, message, data);
+}
+
+
 static void
 get_options (int argc, char **argv)
 {
     int i;
+    gboolean debug = FALSE;
 
     for (i = 1; i < argc; i++) {
         char *arg = argv[i];
 
-        if (strcmp (arg, "-v") == 0 ||
+        if (strcmp (arg, "-d") == 0 ||
+            strcmp (arg, "--debug") == 0) {
+            debug = TRUE;
+        }
+        else if (strcmp (arg, "-v") == 0 ||
             strcmp (arg, "--version") == 0) {
             version ();
             exit (0);
@@ -358,7 +419,8 @@ get_options (int argc, char **argv)
             usage (FALSE);
             exit (0);
         }
-        else if (strcmp (arg, "--help-all") == 0) {
+        else if (strcmp (arg, "--help-all") == 0 ||
+                 strcmp (arg, "--help-gtk") == 0) {
             usage (TRUE);
             exit (0);
         }
@@ -370,6 +432,9 @@ get_options (int argc, char **argv)
             default_device = arg;
         }
     }
+   
+    if (!debug)
+        g_log_set_default_handler (log_cb, NULL);
 }
 
 
@@ -378,7 +443,7 @@ main(int argc, char **argv)
 {
     g_thread_init (NULL);
     gtk_init (&argc, &argv);
-    
+
     get_options (argc, argv);
 
     book = book_new ();
@@ -391,12 +456,12 @@ main(int argc, char **argv)
     g_signal_connect (ui, "stop-scan", G_CALLBACK (cancel_cb), NULL);
     g_signal_connect (ui, "save", G_CALLBACK (save_cb), NULL);
     g_signal_connect (ui, "email", G_CALLBACK (email_cb), NULL);
-    g_signal_connect (ui, "print", G_CALLBACK (print_cb), NULL);
     g_signal_connect (ui, "quit", G_CALLBACK (quit_cb), NULL);
 
     scanner = scanner_new ();
     g_signal_connect (G_OBJECT (scanner), "ready", G_CALLBACK (scanner_ready_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "update-devices", G_CALLBACK (update_scan_devices_cb), NULL);
+    g_signal_connect (G_OBJECT (scanner), "authorize", G_CALLBACK (authorize_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "got-page-info", G_CALLBACK (scanner_page_info_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "got-line", G_CALLBACK (scanner_line_cb), NULL);
     g_signal_connect (G_OBJECT (scanner), "image-done", G_CALLBACK (scanner_image_done_cb), NULL);
